import time
from enum import Enum
from typing import List, Callable, Optional

from langchain_community.embeddings import OpenAIEmbeddings

from lemon_rag.lemon_runtime import models
from lemon_rag.utils import log


class EmbeddingModel(str, Enum):
    DEFAULT = "text-embedding-ada-002"


Embedding_F = Callable[[List[str]], List[List[float]]]


def get_embedding_function(metrics_name: str) -> Embedding_F:
    embeddings = [OpenAIEmbeddings()]

    def init_embeddings_client():
        if embeddings[0]:
            return
        openai_config: Optional[models.OPENAIConfig] = models.OPENAIConfig.get_or_none()
        if not openai_config:
            raise ValueError("openai_config not found")
        embeddings[0] = OpenAIEmbeddings(openai_api_base=openai_config.base_url, openai_api_key=openai_config.api_key)

    def inner(text: List[str]) -> List[List[float]]:
        init_embeddings_client()
        start_time = time.time()
        client = embeddings[0]
        time_cost = time.time() - start_time
        embedding_key_len = [len(k) for k in text]
        log.info("[Embedding] /%s %.3fs embedding_key_len: %s", metrics_name, time_cost, embedding_key_len)
        return client.embed_documents(text)

    return inner



from typing import Set

from pydantic import BaseModel


class RuntimeConfig(BaseModel):
    max_file_bytes: int = 1024 * 1024 * 4
    supported_file_types: Set[str] = {"txt"}


runtime_config: RuntimeConfig = RuntimeConfig()



from functools import partial
from typing import Optional, List

from pydantic import BaseModel
from pymilvus import Connections, Collection, FieldSchema, DataType, CollectionSchema

from lemon_rag.lemon_runtime import models
from lemon_rag.llm.client.base_client import get_embedding_function

default_index_schema = {
    "index_type": "IVF_FLAT",
    "metric_type": "L2",
    "params": {"nlist": 128},
}

embed_sentence = get_embedding_function("embed_knowledge_base_sentence")


class VectorAccess(BaseModel):
    connections: Optional[Connections] = None
    knowledge_base_collection: Optional[Collection] = None
    initialized: bool = False
    init_msg: str = ""

    class Config:
        arbitrary_types_allowed = True

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def _init(self):
        if self.initialized:
            if self.init_msg:
                raise ValueError(f"milvus init failed for {self.init_msg}")
            return

        milvus_token: Optional[models.VectorStoreConfig] = models.VectorStoreConfig.select().get_or_none()
        if not milvus_token:
            self.initialized = True
            self.init_msg = "no milvus config found"
            raise ValueError("no milvus config found")
        connections = Connections()
        connections.connect("default", uri=milvus_token.uri, token=milvus_token.token)
        self.connections = connections
        self._init_collections()
        self.initialized = True

    def _init_collections(self):
        self.knowledge_base_collection = Collection(
            "data_model_overview",
            schema=CollectionSchema(
                [
                    FieldSchema(name="id", dtype=DataType.INT32, is_primary=True),
                    FieldSchema(name="raw_content", dtype=DataType.VARCHAR, max_length=2048),
                    FieldSchema(name="doc_name", dtype=DataType.VARCHAR, max_length=256),
                    FieldSchema(name="doc_id", dtype=DataType.INT32),
                    FieldSchema(name="vector", dtype=DataType.FLOAT_VECTOR, dim=1536),
                ]
            ))
        if not self.knowledge_base_collection.has_index():
            self.knowledge_base_collection.create_index("vector", default_index_schema)
        self.knowledge_base_collection.load()

    def save_sentence(self, sentence_record: models.KnowledgeSentence, vector: List[float]) -> None:
        self.knowledge_base_collection.insert([
            {
                "id": sentence_record.id,
                "raw_content": sentence_record.raw_content,
                "doc_name": sentence_record.paragraph.file.filename,
                "doc_id": sentence_record.paragraph.file.id,
                "vector": vector
            }
        ])


vector_access = VectorAccess()



import hashlib


def hash_password(password: str) -> str:
    # Hash the password using SHA-256 algorithm
    hashed_password = hashlib.sha256(password.encode('utf-8')).hexdigest()

    return hashed_password
